@@ -1,7 +1,7 @@
 /*
  * song.cpp - root of the model tree
  *
- * Copyright (c) 2004-2010 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2011 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -31,8 +31,8 @@
 #include <math.h>
 
 #include "song.h"
-#include "automation_track.h"
-#include "automation_editor.h"
+#include "AutomationTrack.h"
+#include "AutomationEditor.h"
 #include "bb_editor.h"
 #include "bb_track.h"
 #include "bb_track_container.h"
@@ -68,7 +68,7 @@ tick_t midiTime::s_ticksPerTact = DefaultTicksPerTact;
 
 song::song() :
 	trackContainer(),
-	m_globalAutomationTrack( dynamic_cast<automationTrack *>(
+	m_globalAutomationTrack( dynamic_cast<AutomationTrack *>(
 				track::create( track::HiddenAutomationTrack,
 								this ) ) ),
 	m_tempoModel( DefaultTempo, MinTempo, MaxTempo, this, tr( "Tempo" ) ),
@@ -343,7 +343,7 @@ void song::processNextBuffer()
 	bool check_loop = tl != NULL && m_exporting == false &&
 				tl->loopPointsEnabled() &&
 				!( m_playMode == Mode_PlayPattern &&
-					m_patternToPlay->freezing() == true );
+					m_patternToPlay->isFreezing() == true );
 	if( check_loop )
 	{
 		if( m_playPos[m_playMode] < tl->loopBegin() ||
@@ -467,11 +467,22 @@ void song::processNextBuffer()
 
 
 
+bool song::isFreezingPattern() const
+{
+	return isPlaying() &&
+				m_playMode == Mode_PlayPattern &&
+				m_patternToPlay != NULL &&
+				m_patternToPlay->isFreezing();
+}
+
+
+
+
 bool song::realTimeTask() const
 {
 	return !( m_exporting == true || ( m_playMode == Mode_PlayPattern &&
 		  	m_patternToPlay != NULL &&
-			m_patternToPlay->freezing() == true ) );
+			m_patternToPlay->isFreezing() == true ) );
 }
 
 
@@ -708,9 +719,9 @@ bpm_t song::getTempo()
 
 
 
-automationPattern * song::tempoAutomationPattern()
+AutomationPattern * song::tempoAutomationPattern()
 {
-	return automationPattern::globalAutomationPattern( &m_tempoModel );
+	return AutomationPattern::globalAutomationPattern( &m_tempoModel );
 }
 
 
@@ -744,14 +755,20 @@ void song::clearProject()
 
 	engine::fxMixer()->clear();
 
-	if( engine::getAutomationEditor() )
+	if( engine::automationEditor() )
 	{
-		engine::getAutomationEditor()->setCurrentPattern( NULL );
+		engine::automationEditor()->setCurrentPattern( NULL );
 	}
-	automationPattern::globalAutomationPattern( &m_tempoModel )->clear();
-	automationPattern::globalAutomationPattern( &m_masterVolumeModel )->
+
+	m_tempoModel.reset();
+	m_masterVolumeModel.reset();
+	m_masterPitchModel.reset();
+	m_timeSigModel.reset();
+
+	AutomationPattern::globalAutomationPattern( &m_tempoModel )->clear();
+	AutomationPattern::globalAutomationPattern( &m_masterVolumeModel )->
 									clear();
-	automationPattern::globalAutomationPattern( &m_masterPitchModel )->
+	AutomationPattern::globalAutomationPattern( &m_masterPitchModel )->
 									clear();
 
 	engine::getMixer()->unlock();
@@ -933,10 +950,10 @@ void song::loadProject( const QString & _file_name )
 							node.toElement() );
 				}
 				else if( node.nodeName() ==
-					engine::getAutomationEditor()->
+					engine::automationEditor()->
 								nodeName() )
 				{
-					engine::getAutomationEditor()->
+					engine::automationEditor()->
 						restoreState( node.toElement() );
 				}
 				else if( node.nodeName() ==
@@ -968,7 +985,7 @@ void song::loadProject( const QString & _file_name )
 	ControllerConnection::finalizeConnections();
 
 	// resolve all IDs so that autoModels are automated
-	automationPattern::resolveAllIDs();
+	AutomationPattern::resolveAllIDs();
 
 
 	engine::getMixer()->unlock();
@@ -977,6 +994,8 @@ void song::loadProject( const QString & _file_name )
 
 	engine::projectJournal()->setJournalling( true );
 
+	emit projectLoaded();
+
 	m_loadingProject = false;
 	m_modified = false;
 
@@ -984,17 +1003,11 @@ void song::loadProject( const QString & _file_name )
 	{
 		engine::mainWindow()->resetWindowTitle();
 	}
-	if( engine::getSongEditor() )
-	{
-		engine::getSongEditor()->scrolled( 0 );
-	}
 }
 
 
-
-
-// save current song
-bool song::saveProject()
+// only save current song as _filename and do nothing else
+bool song::saveProjectFile( const QString & _filename )
 {
 	multimediaProject mmp( multimediaProject::SongProject );
 
@@ -1003,7 +1016,6 @@ bool song::saveProject()
 	m_masterVolumeModel.saveSettings( mmp, mmp.head(), "mastervol" );
 	m_masterPitchModel.saveSettings( mmp, mmp.head(), "masterpitch" );
 
-
 	saveState( mmp, mmp.content() );
 
 	m_globalAutomationTrack->saveState( mmp, mmp.content() );
@@ -1012,7 +1024,7 @@ bool song::saveProject()
 	{
 		engine::getControllerRackView()->saveState( mmp, mmp.content() );
 		engine::getPianoRoll()->saveState( mmp, mmp.content() );
-		engine::getAutomationEditor()->saveState( mmp, mmp.content() );
+		engine::automationEditor()->saveState( mmp, mmp.content() );
 		engine::getProjectNotes()->
 			SerializingObject::saveState( mmp, mmp.content() );
 		m_playPos[Mode_PlaySong].m_timeLine->saveState(
@@ -1021,8 +1033,17 @@ bool song::saveProject()
 
 	saveControllerStates( mmp, mmp.content() );
 
+    return mmp.writeFile( _filename );
+}
+
+
+
+// save current song and update the gui
+bool song::guiSaveProject()
+{
+	multimediaProject mmp( multimediaProject::SongProject );
 	m_fileName = mmp.nameWithExtension( m_fileName );
-	if( mmp.writeFile( m_fileName ) == true && engine::hasGUI() )
+	if( saveProjectFile( m_fileName ) && engine::hasGUI() )
 	{
 		textFloat::displayMessage( tr( "Project saved" ),
 					tr( "The project %1 is now saved."
@@ -1049,12 +1070,12 @@ bool song::saveProject()
 
 
 // save current song in given filename
-bool song::saveProjectAs( const QString & _file_name )
+bool song::guiSaveProjectAs( const QString & _file_name )
 {
 	QString o = m_oldFileName;
 	m_oldFileName = m_fileName;
 	m_fileName = _file_name;
-	if( saveProject() == false )
+	if( guiSaveProject() == false )
 	{
 		m_fileName = m_oldFileName;
 		m_oldFileName = o;
